require 'cocoapods'
require_relative 'helper/feature_switches'

module Pod
  class Source
    class Manager
      # 二进制源
      def binary_source
        source_with_name_or_url("http://gitlab.dushuclub.io/cocoapods/HDBinarySpecs.git")
      end

      # 三方库源
      def third_party_source
        source_with_name_or_url("http://gitlab.dushuclub.io/cocoapods/Specs.git")
      end

      # 私有库源
      def private_source
        source_with_name_or_url("http://gitlab.dushuclub.io/cocoapods/HDSourceSpecs.git")
      end

    end
  end
end

module Pod
  class Resolver

    # 用二进制源查找name和version，存在就用新的ResolverSpecification
    # 如果找不到返回原始的ResolverSpecification
    old_resolver_specs_by_target = instance_method(:resolver_specs_by_target)
    define_method(:resolver_specs_by_target) do
      # Pod::UI.puts "Cocoapods-bone resolver_specs_by_target ------ "
      specs_by_target = old_resolver_specs_by_target.bind(self).call
      if Podfile::DSL.is_all_binary == nil || Podfile::DSL.is_all_binary == false
        return specs_by_target
      end
      # 只有在pod install的时候执行替换源，其他逻辑不参与
      if Pod.is_ignore_hook_install == nil || Pod.is_ignore_hook_install == true
        specs_by_target.each do |target, rspecs|
          specs_by_target[target] = rspecs.map do |rspec|
            sources_manager = Config.instance.sources_manager
            binary_source = sources_manager.binary_source
            third_party_source = sources_manager.third_party_source
            private_source = sources_manager.private_source
            spec_version = rspec.spec.version

            if Podfile::DSL.source_pods.include?(rspec.name)

              # Pod::UI.puts "name:#{rspec.name}
              #             rspec.source：#{rspec.source.url}
              #             third_party_source:#{third_party_source.url}
              #             private_source: #{private_source.url} \n"

              if rspec.source.url != third_party_source.url && rspec.source.url != private_source.url
                used_by_only = rspec.used_by_non_library_targets_only
                begin
                  specification = third_party_source.specification(rspec.name, spec_version)
                  # Pod::UI.puts "specification : -----------#{specification}"
                  rspec = ResolverSpecification.new(specification, used_by_only, third_party_source)
                rescue Exception => e
                  rspec = ResolverSpecification.new(rspec.spec, used_by_only, private_source)
                end
              end
            else
              begin

                specification = binary_source.specification(rspec.root.name, spec_version)
                # Pod::UI.puts "specification----- #{specification}, #{used_by_only}, #{binary_source}"

                if rspec.spec.subspec?
                  specification = specification.subspec_by_name(rspec.name, false, true)
                end

                next unless specification

                used_by_only = rspec.used_by_non_library_targets_only

                rspec = ResolverSpecification.new(specification, used_by_only, binary_source)
              rescue Exception => e
                rspec
              end
            end

            rspec
          end.compact
        end
      else
        #制作二进制库，需要将源强制切换到源码源
        Pod::UI.puts "制作二进制库，force switch source pods"
        specs_by_target.each do |target, rspecs|

          specs_by_target[target] = rspecs.map do |rspec|
            sources_manager = Config.instance.sources_manager
            third_party_source = sources_manager.third_party_source
            private_source = sources_manager.private_source
            spec_version = rspec.spec.version

            used_by_only = rspec.used_by_non_library_targets_only

            begin
              specification = third_party_source.specification(rspec.name, spec_version)
              rspec = ResolverSpecification.new(specification, used_by_only, third_party_source)
            rescue Exception => e
              rspec = ResolverSpecification.new(rspec.spec, used_by_only, private_source)
            end
            rspec
          end.compact
        end
      end

      specs_by_target
    end
  end
end

