require 'cocoapods'
require 'pathname'
require 'fileutils'
require 'cocoapods-hd/upload/constant'
require_relative 'spec_source_creator'
require 'net/ftp'

module CocoapodsHd
  class UploadHelper

    attr_accessor :code_spec, :name, :version, :file_path, :platforms

    def initialize(code_spec = nil, file_path = nil)
      @code_spec = code_spec
      @platforms = ['ios']
      @file_path = file_path
      @version = code_spec.version
      @name = code_spec.root.name

      puts "基础信息： name: #{name} --- version: #{version} --- file_path: #{file_path} --- platforms: #{platforms}  "

    end

    def spec_creator
      spec_creator = SpecSourceCreator.new(code_spec, name, file_path)
      spec_creator.create
      spec_creator.write_spec_file
      spec_creator.filename
    end

    def upload
      # 上传zip包
      upload_static_zip
      # push binary repo podspec.json
      push_binary_repo
    end

    def upload_static_zip
      puts "开始上传zip --------- "

      create_dir_command = "curl -u #{Constant.user_and_password} #{Constant.ftp_url}/#{@name}/#{@version.to_s}/ --ftp-create-dirs"
      upload_zip_command = "curl -u #{Constant.user_and_password} #{Constant.ftp_url}/#{@name}/#{@version.to_s}/ -T #{file_path}/#{@name}.zip"

      `#{create_dir_command}`
      `#{upload_zip_command}`
      # 查看curl返回结果
      # create_dir_result = %x(#{create_dir_command})
      # puts "create_dir_result 执行结果 : #{create_dir_result} --- "

      STDERR.puts "zip upload success ".green
    end

    # 上传二进制 podspec
    def push_binary_repo
      # 创建目录 ps: AFNetworking/4.0.1
      add_spec_to_repo

      # push spec to origin gitlab
      `git status -s`
      `git add .`
      `git commit -m '添加 binary #{@name} #{@version.to_s}'`
      `git push origin master`
      STDERR.puts "#{name}.podspec.json psuh success ".green

    end

    def add_spec_to_repo
      podspec_name = "#{@name}.podspec.json"
      podspec_json = File.join(Dir.pwd, podspec_name)

      FileUtils.chdir(binary_repo_dir)
      spec_path = File.join(@name, @version.to_s)

      FileUtils.mkdir_p(spec_path) unless File.exist?(spec_path)

      # 移动podspec.json到本地repo
      FileUtils.move(podspec_json, spec_path)
    end

    # 获取 HDBinarySpecs 路径
    def binary_repo_dir
      unless File.directory?(Constant.binary_repo)
        `pod repo add HDBinarySpecs #{Constant.binary_url}`
        # 删除多余的repo
        FileUtils.rm_rf(Constant.unused_binary_repo)
      end
      Constant.binary_repo
    end

  end
end